#!/bin/bash

# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Frantisek Reznicek <freznice@redhat.com>
#         David Sommerseth <dsommers@redhat.com>



                    ##### Function Declarations #####

# ###########################################################################
# ###########################################################################
#                              Logging functions
# ###########################################################################
# ###########################################################################

# variables
STARTOFLINE=0
LOG_TO_FILE=""

# ###########################################################################
# log() - prints string to stdout and optionally to an file without \n
#         file can be specified in LOG_TO_FILE variable
# ###########################################################################
function log() 
{
  if [ "${STARTOFLINE}" == "0" ]; then
    tstamp="[`date +%H:%M:%S`]"
  else 
    tstamp=""
  fi
  # log to stdout
  printf "${tstamp} $1"
  # log to file
  if [ -n "${LOG_TO_FILE}" -a  -w "${LOG_TO_FILE}" ]; then
    printf "${tstamp} $1" >> ${LOG_TO_FILE}
  fi
  
  STARTOFLINE=1
}

# ###########################################################################
# lognl() - prints string to stdout and optionally to an file with \n
#           file can be specified in LOG_TO_FILE variable
# ###########################################################################
function lognl() 
{
  # log to stdout & file
  log "$1"
  echo
  if [ -n "${LOG_TO_FILE}" -a  -w "${LOG_TO_FILE}" ]; then
    echo >> ${LOG_TO_FILE}
  fi
  
  STARTOFLINE=0
}

# ###########################################################################
# lognl_ct() - prints string to stdout and to common transcript with \n
#              file can be specified in COMMON_TRANSCRIPT
# ###########################################################################
function lognl_ct()
{
  # time stamp
  tstamp="[$(date +%H:%M:%S)]"
  
  # log to stdout
  echo "${tstamp} $1"
  
  # log to file
  if [ -n "${COMMON_TRANSCRIPT}" -a -w "${COMMON_TRANSCRIPT}" ]; then
    echo "${tstamp} $1" >> ${COMMON_TRANSCRIPT}
  fi
  
}

# another logging functions used by David
# ###########################################################################

# ###########################################################################
# Log ( [text] )
# prints the text with the time stamp to stdout
# ###########################################################################
function Log 
{
  echo -e ":: [`date +%H:%M:%S`] :: $1" 
}

# ###########################################################################
# HeaderLog ( [text] )
# prints the text with the time stamp to stdout in the header
# ###########################################################################
function HeaderLog 
{
  echo -e "\n:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::"
  echo -e ":: [`date +%H:%M:%S`] :: $1" 
  echo -e ":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
}


# ###########################################################################
# ###########################################################################
#                          RHTS (wrapping) functions
# ###########################################################################
# ###########################################################################

# ###########################################################################
# rhts_load_environment( [quiet] )
#   detects whether running in RHTS environment
#   returns 0/1 if under RHTS / outside RHTS
#   exports RHTS_TEST_ENVIRONMENT to 1 / 0
# ###########################################################################
function rhts_load_environment ()
{
  # vars
  rhts_environment="/usr/bin/rhts-environment.sh"
  
  # detect whether global test variables are defined
  if [ -z "${VERSION}" ]; then
    VERSION=1.0                  # test version
  fi
  if [ -z "${RESULT}" ]; then
    RESULT=FAIL                  # FAIL unless test explicitly passes
  fi
  if [ -z "${ERR_CNT}" ]; then
    ERR_CNT=0                    # error counter init
  fi
  if [ -z "${TST_CNT}" ]; then
    TST_CNT=0                    # test count
  fi
  if [ -z "${OUTPUTFILE}" ]; then
    OUTPUTFILE="$PWD/$(basename $PWD).log"
  fi
  if [ -z "${VERBOSE_LEVEL}" ]; then
    VERBOSE_LEVEL=${TEST_PARAM_RHTS_TEST_VERBOSE_LEVEL:-1}
  fi
  
  # source the RHTS environment
  if [ -r ${rhts_environment} ]; then
    source ${rhts_environment}
    if [ "$1" != "quiet" ]; then
      lognl "rhts_load_environment:Running in RHTS mode ${rhts_environment} loaded"
    fi
    RHTS_TEST_ENVIRONMENT=1
    return 0
  else
    if [ "$1" != "quiet" ]; then
      lognl "rhts_load_environment:Running outside RHTS mode!"
    fi
    RHTS_TEST_ENVIRONMENT=0
    return 1
  fi
}

# ###########################################################################
# rhts_report_result( ... )
#   RHTS's report_result function wrapper
# ###########################################################################
function rhts_report_result ()
{
  # vars
  ecode=0
  rhts_environment="/usr/bin/rhts-environment.sh"
  # logging
  rep_str="Test Summary: TESTS:${TST_CNT}, FAILS:${ERR_CNT}, DURATION:${SECONDS}sec"
  lognl "$(gawk -v s="${rep_str}" 'BEGIN{for(i=0;i<length(s);i++){printf("-")}}')"
  lognl "${rep_str}"
  lognl "$(gawk -v s="${rep_str}" 'BEGIN{for(i=0;i<length(s);i++){printf("-")}}')"
  # running report_result
  if [ -r ${rhts_environment} -a "${RHTS_TEST_ENVIRONMENT}"=="1" ]; then
    report_result $*
    ecode="$?"
  fi
  # return value
  return ${ecode}
}

# ###########################################################################
# ###########################################################################
#                       Local watchdog functions
# ###########################################################################
# ###########################################################################

# watchdog variables
WATCHDOG_START=0                  # starting wd mark
WATCHDOG_TIMEOUT=0                # wd timeout/period
WATCHDOG_LEFT=0                   # output result - duration remaining to TO


# ###########################################################################
# set_watchdog( timeout )
# set_watchdog_timeout( timeout )
#   set the watchdog to a number - set WATCHDOG timeout to a duration [sec]
#                                - initializes and runs the watchdog
# ###########################################################################
function set_watchdog ()
{
  if [ -z "$1" -a "$1" -ge 0 ]; then
    return 1
  else
    WATCHDOG_START=${SECONDS}
    WATCHDOG_TIMEOUT=$1
    return 0
  fi
}

function set_watchdog_timeout ()
{
  set_watchdog $*
  return $?
}


# ###########################################################################
# get_watchdog ()
# get_watchdog_state ()
#   get the watchdog state, returns the number of remaining ticks
#   as return value and also sets variable WATCHDOG_LEFT
# ###########################################################################

function get_watchdog ()
{
  if [ "$((${WATCHDOG_START}+${WATCHDOG_TIMEOUT}))" -lt "${SECONDS}"  ]; then
    WATCHDOG_LEFT=0
    return 0
  else
    WATCHDOG_LEFT=$((${WATCHDOG_START}+${WATCHDOG_TIMEOUT}-${SECONDS}))
    if [ "${WATCHDOG_LEFT}" -gt 255 ]; then
      return 255
    else
      return ${WATCHDOG_LEFT}
    fi
  fi
  return $?
}

function get_watchdog_state ()
{
  get_watchdog $*
  return $?
}






# ###########################################################################
# ###########################################################################
#                               Misc. functions
# ###########################################################################
# ###########################################################################


# ###########################################################################
# Exit() function
#   synchronizes all running parent processes using file presence
# ###########################################################################
EXITING_SIGNAL_FILE="exit.signal.log"
function Exit ()
{
  touch ${EXITING_SIGNAL_FILE}
  exit $*
}

# ###########################################################################
# trasher ()
#   trasher function removes file/directory if exists
#   returns number of deleted files/directories
# ###########################################################################
function trasher ()
{
  item_cnt=0
  if [ -n "$*" ]; then
    for i_item in $*; do                    # main loop
      if [ -e ${i_item} ]; then             # existency test
        if [ -d ${i_item} ]; then
          rm -rf ${i_item}                  # a directory
        else
          rm -f ${i_item}                   # a file
        fi
        let "item_cnt++"
      fi
    done
  fi
  return ${item_cnt}
}

# ###########################################################################
# wait_till_file_exist ( <file/dir_name> [<timeout>==10] )
#   wait_till_file_exist waits for existence of file/dir for certain timeout
#   returns 0/1 ~ file/dir exists/created within timeout / not created within to
# ###########################################################################
function wait_till_file_exist ()
{
  
  if [ -z "$1" ]; then
    lognl "wait_till_file_exist: No parameters given. Stop."
    return 1
  else
    # param specified
    if [ -e "$1" ]; then
      lognl "wait_till_file_exist: File/dir ($1) exists already."
      return 0
    else
      # assign timeout
      if [ -z "$2" ]; then
        c_to="10"
      else
        c_to="$2"
      fi
      # get current timestamp
      c_ts=${SECONDS}
      while [ 1 ]; do
        if [ -e "$1" -o \
             "$((${SECONDS}-${c_ts}))" -gt ${c_to} ]; then
          break
        fi
      done
      
      # get result
      if [ -e "$1" ] ;then
        lognl "wait_till_file_exist: File/dir ($1) created. (dur:$((${SECONDS}-${c_ts}))sec)"
        return 0
      else
        lognl "wait_till_file_exist: File/dir ($1) not created. (dur:$((${SECONDS}-${c_ts}))sec)"
        return 1
      fi
    
    fi
  fi
  
}

# ###########################################################################
# wait_till_file_not_exist ( <file/dir_name> [<timeout>==10] )
#   wait_till_file_not_exist waits for file/dir removal for certain timeout
#   returns 0/1 ~ file/dir not exist/going deleted within timeout
# ###########################################################################
function wait_till_file_not_exist ()
{
  
  if [ -z "$1" ]; then
    lognl "wait_till_file_not_exist: No parameters given. Stop."
    return 1
  else
    # param specified
    if [ -e "$1" ]; then
      
      # assign timeout
      if [ -z "$2" ]; then
        c_to="10"
      else
        c_to="$2"
      fi
     
      # get current timestamp
      c_ts=${SECONDS}
      while [ 1 ]; do
        if [ ! -e "$1" -o \
             "$((${SECONDS}-${c_ts}))" -gt ${c_to} ]; then
          break
        fi
      done
      
      # get result
      if [ -e "$1" ] ;then
        lognl "wait_till_file_exists: File/dir ($1) not deleted. (dur:$((${SECONDS}-${c_ts}))sec)"
        return 1
      else
        lognl "wait_till_file_exists: File/dir ($1) deleted. (dur:$((${SECONDS}-${c_ts}))sec)"
        return 0
      fi
    else
      lognl "wait_till_file_not_exist: File/dir ($1) already deleted."
      return 0
    fi  
  fi
  
}

# ###########################################################################
# get_qpidd_version ()
#   gets qpidd version and fills it to QPIDD_VER variable
#   returns qpidd version without dot and heading zeros
# ###########################################################################
MRG_CPP_BROKER="qpidd"

function get_qpidd_version ()
{
  
  if [ -x ${QPIDD_DIR}/${MRG_CPP_BROKER} ]; then
    QPIDD_VER="$(${QPIDD_DIR}/${MRG_CPP_BROKER} --version | awk '{for(i=1;i<NF;i++){if(tolower($i)=="version"){printf $(i+1)}}}')"
  else
    qpidd_which_str="$(which ${MRG_CPP_BROKER} 2> /dev/null)"
    if [ "$?" == "0" ]; then
      # qpidd found in $PATH
      qpidd_which_str="$(which ${MRG_CPP_BROKER} 2> /dev/null | tail -1)"
      QPIDD_VER="$(${qpidd_which_str} --version | awk '{for(i=1;i<NF;i++){if(tolower($i)=="version"){printf $(i+1)}}}')"
    else
      # qpidd not found in $PATH, testing /usr/sbin
      if [ -x /usr/sbin/${MRG_CPP_BROKER} ]; then
        # qpidd found in /usr/sbin
        QPIDD_VER="$(/usr/sbin/${MRG_CPP_BROKER} --version | awk '{for(i=1;i<NF;i++){if(tolower($i)=="version"){printf $(i+1)}}}')"
      else
        # qpidd neither found in $PATH nor in /usr/sbin 
        if [ -n "${RHTS_TEST_MRG_COMPILED_DIR}" -a \
          -x ${RHTS_TEST_MRG_COMPILED_DIR}/sbin/${MRG_CPP_BROKER} ]; then
          # qpidd found compiled & installed
          QPIDD_VER="$(${RHTS_TEST_MRG_COMPILED_DIR}/sbin/${MRG_CPP_BROKER} --version | awk '{for(i=1;i<NF;i++){if(tolower($i)=="version"){printf $(i+1)}}}')"
        else
          # qpidd not found at all
          QPIDD_VER="0.0"
        fi
      fi
    fi
  fi
  
  # export qpidd version
  export QPIDD_VER
  
  # return just numbers without heading zeros
  return "$(echo ${QPIDD_VER} | tr -d . | awk '{printf(int($1+0))}')"
}




# ###########################################################################
# ###########################################################################
#                             Install functions
# ###########################################################################
# ###########################################################################


# ###########################################################################
# check_tool_install      ( tool_name, ... )
# check_tool_availibility ( tool_name, ... )
#   checks whether given tool is installed in path
#   if all installed returns 0, if not returns 1
# ###########################################################################
function check_tool_install ()
{
  result=0                         # return code
  
  pkgs="$*"                        # read in packages
  for i_pkg in ${pkgs}; do
    which ${i_pkg} > /dev/null 2>&1
    if [ "$?" == "0" ]; then
      lognl "check_tool_install:tool ${i_pkg} found in $(which ${i_pkg} | head -1)"
    else
      lognl "check_tool_install:tool ${i_pkg} not found in \${PATH}"
      let "result++"
    fi
  done
  return ${result}                # return number of not installed packages
}

# alias function
function check_tool_availability ()
{
  check_tool_install $*
  return $?
}

# ###########################################################################
# fetch_repofile( url_to_repofile, target_repofile )
#   downloads repofile and enable it by default and store it under /etc/yum.repos.d/
# ###########################################################################
function fetch_repofile () 
{
  repof="$1"
  destf="$2"

  tmp=$(mktemp /tmp/fetchrepo.XXXXX)
  repo_path=/etc/yum.repos.d
  
  log "fetch_repofile:Fetching repo-file: ${repof} ... "
  curl ${repof}  2> ${tmp}.err >${tmp}
  if [ "$?" != 0 ]; then
    lognl "fetch_repofile:Failed!"
    lognl "----- curl error -----------------------------"
    cat ${tmp}.err
    lognl "----------------------------------------------"
    rm -f ${tmp}*
    return 1
  fi

  # enabling repofile
  cat ${tmp} | sed "s/enabled=0/enabled=1/" > ${repo_path}/${destf}
  
  # clean-up
  rm -f ${tmp}*
  
  # final check
  if [ -r /etc/yum.repos.d/${destf} ]; then
    lognl "fetch_repofile: repofile installed to ${repo_path}/${destf} (enabled)"
    return 0
  else
    lognl "fetch_repofile:ERROR:repofile not installed (${repo_path}/${destf} not found)"
    return 1
  fi
}



# ###########################################################################
# install_package_yum ( pkg, ... )
#   performs yum package[s] install one-by-one
# ###########################################################################
function install_package_yum ()
{
  
  result=0                         # return code
  
  pkgs="$*"                        # read in packages
  for i_pkg in ${pkgs}; do
    # query package
    rpm -q ${i_pkg} 2> /dev/null 1>&2
    if [ "$?" != "0" ]; then
      # package not found - installing
      lognl "install_package_yum:installing package ${i_pkg}"
      yum -y install ${i_pkg}
      
      if [ "$?" != "0" ]; then     # package install failed (A) ?
        let "result++"
      fi
    
      # query package again
      rpm -q ${i_pkg} 2> /dev/null 1>&2
      
      if [ "$?" != "0" ]; then     # package install failed (B) ?
        let "result++"
      fi
      
      if [ "${result}" != "0" ]; then
        lognl "install_package_yum:ERROR:package ${i_pkg} install failed!"
      fi
      
    else
      lognl "install_package_yum:package ${i_pkg} already installed"
    fi
  done
  
  return ${result}
}

# ###########################################################################
# upgrade_package_yum ( pkg, ... )
#   performs yum package[s] upgrade one-by-one
# ###########################################################################
function upgrade_package_yum ()
{
  
  result=0                         # return code
  
  pkgs="$*"                        # read in packages
  for i_pkg in ${pkgs}; do
    # query package
    pkg_ver_before="$(rpm -q ${i_pkg})"
    rpm -q ${i_pkg} 2> /dev/null 1>&2
    if [ "$?" != "0" ]; then
      # package not found - installing
      lognl "upgrade_package_yum:installing package ${i_pkg}"
      yum -y install ${i_pkg}
      
      if [ "$?" != "0" ]; then     # package install failed (A) ?
        let "result++"
      fi
    
      # query package again
      rpm -q ${i_pkg} 2> /dev/null 1>&2
      
      if [ "$?" != "0" ]; then     # package install failed (B) ?
        let "result++"
      fi
      
      if [ "${result}" != "0" ]; then
        lognl "upgrade_package_yum:ERROR:package ${i_pkg} install failed!"
      fi
      
    else
      lognl "upgrade_package_yum:upgrading package ${i_pkg} already installed"
      # package upgrade
      yum -y upgrade ${i_pkg}
      ecode="$?"
      
      # check new version
      pkg_ver_after="$(rpm -q ${i_pkg})"
      
      if [ "$?" != "0" ]; then     # package upgrade failed ?
        let "result++"
        lognl "upgrade_package_yum:ERROR:package ${i_pkg} upgrade failed"
      else
        lognl "upgrade_package_yum:package ${i_pkg} upgrade OK (${pkg_ver_before} -> ${pkg_ver_after})"
      fi
      
    fi
  done
  
  return ${result}
}


# ###########################################################################
# install_packages_yum ( pkg, ... )
#   performs yum package[s] install as the group in one step
# ###########################################################################
function install_packages_yum ()
{
  if [ "$*" == "" ]; then
    lognl "install_packages_yum:no package to install (blank argument)"
    return 0
  else
    lognl "install_packages_yum:installing packages $*"
    
    # yum group install in one step
    yum -y install $*
    ecode="$?"
    
    if [ "${ecode}" == "0" ]; then
      lognl "install_packages_yum:Installation succeeded"
    else
      lognl "install_packages_yum:ERROR:Installation failed"
    fi
    # return yum exit code
    return ${ecode}
  fi
}

# ###########################################################################
# upgrade_packages_yum ( pkg, ... )
#   performs yum package[s] upgrade as the group in one step
# ###########################################################################
function upgrade_packages_yum ()
{
  if [ "$*" == "" ]; then
    lognl "upgrade_packages_yum:no package to upgrade (blank argument)"
    return 0
  else
    lognl "upgrade_packages_yum:upgrading packages $*"
    
    # yum group install in one step
    yum -y upgrade $*
    ecode="$?"
    
    if [ "${ecode}" == "0" ]; then
      lognl "upgrade_packages_yum:Upgrade succeeded"
    else
      lognl "upgrade_packages_yum:ERROR:Upgrade failed"
    fi
    # return yum exit code
    return ${ecode}
  fi
}
# ###########################################################################
# install_source_tarball ( pkg_url, ... )
#   performs source tar-ball installation of packages
#
#   multiple alternative package locations can be specified, first one wins
#   the rest are skipped if first one is downloaded OK
# ###########################################################################
function install_source_tarball ()
{
  # vars
  result=0
  i_dwn_to=30
  i_dwn_tries=2
  i_dwn_success_bn=""
  i_dwn_failed_bn=""
  
  # browsing packages
  for i_pkg_url in $*; do
    
    # vars
    i_pkg_bn="$(basename ${i_pkg_url})"
    i_pkg_dn="$(dirname ${i_pkg_url})"
    
    # stopping when package ${i_pkg_bn} was not downloaded OK
    if [ -n "${i_dwn_failed_bn}" -a "${i_dwn_failed_bn}" != "${i_pkg_bn}" ]; then
      lognl "..download error (${i_dwn_failed_bn}) Could not be downloaded. Stop!"
      let "result++"
      break
    fi
    
    # skipping next url when package already downloaded
    if [ -n "${i_dwn_success_bn}" -a "${i_dwn_success_bn}" == "${i_pkg_bn}" ]; then
      continue
    fi
    
    lognl "install_source_tarball: Processing ${i_pkg_url}"
    
    # download
    wget --timeout=${i_dwn_to} --tries=${i_dwn_tries} ${i_pkg_url} > /dev/null 2>&1
    # test download
    if [ "$?" == "0" -a -e ${i_pkg_bn} ]; then
      lognl "..downloaded ok (${i_pkg_bn})"
      i_dwn_success_bn="${i_pkg_bn}"
      i_dwn_failed_bn=""
    else
      lognl "..download error (${i_pkg_bn}) Trying once more."
      i_dwn_success_bn=""
      i_dwn_failed_bn="${i_pkg_bn}"
      continue
    fi
    
    # extract
    pkg_type_tbz2="$(echo ${i_pkg_bn} | egrep -c '(.tar.bz2$|.tbz2$)')"
    pkg_type_tgz="$(echo ${i_pkg_bn} | egrep -c '(.tar.gz$|.tgz$)')"
    pkg_type_zip="$(echo ${i_pkg_bn} | egrep -c '(.zip$)')"
    
    if [ "${pkg_type_tbz2}" == "1" ]; then
      extract_tool="tar xjf ${i_pkg_bn}"
      test_archive_tool="tar tjf ${i_pkg_bn}"
    elif [ "${pkg_type_tgz}" == "1" ]; then
      extract_tool="tar xzf ${i_pkg_bn}"
      test_archive_tool="tar tzf ${i_pkg_bn}"
    elif [ "${pkg_type_zip}" == "1" ]; then
      extract_tool="unzip ${i_pkg_bn}"
      test_archive_tool="unzip -t ${i_pkg_bn}"
    else
      lognl "..tarball type unknown"
    fi
    
    # archive testing
    ${test_archive_tool} > /dev/null
    # test archive
    if [ "$?" == "0" ]; then
      lognl "..tar-ball integrity ok (${i_pkg_bn})"
    else
      lognl "..tar-ball integrity error (${i_pkg_bn}) Stop."
      let "result++"
      break
    fi
    
    # extraction core
    ${extract_tool}
    
    # list archive
    if [ "${pkg_type_zip}" == "1" ]; then
      ${test_archive_tool} | awk '{if($$NF=="OK"){print $$2}}' > ${i_pkg_bn}.list
    else
      ${test_archive_tool} > ${i_pkg_bn}.list
    fi
    
    # find installation way
    configure_cnt="$(sort ${i_pkg_bn}.list | grep -c '/configure$' )"
    configure_str="$(sort ${i_pkg_bn}.list | grep '/configure$' | head -1 )"
    
    booststrap_cnt="$(sort ${i_pkg_bn}.list | grep -c '/boostrap$' )"
    booststrap_str="$(sort ${i_pkg_bn}.list | grep '/boostrap$$' | head -1 )"
    
    makefile_cnt="$(sort ${i_pkg_bn}.list | grep -c '/[mM]akefile$' )"
    makefile_str="$(sort ${i_pkg_bn}.list | grep '/[mM]akefile$' | head -1 )"
    
    # compilation core
    if [ "${configure_cnt}" -gt "0" ]; then
      # configure && make & make install
      pushd $(dirname ${configure_str}) > /dev/null
      ./configure && make && make install
      ecode="$?"
      popd > /dev/null
    elif [ "${boostrap_cnt}" -gt "0" ]; then
      # boostrap && configure && make & make install
      pushd $(dirname ${boostrap_str}) > /dev/null
      ./boostrap && ./configure && make && make install
      ecode="$?"
      popd > /dev/null
    elif [ "${makefile_cnt}" -gt "0" ]; then
      # make & make install
      pushd $(dirname ${makefile_str}) > /dev/null
      make && make install
      ecode="$?"
      popd > /dev/null
    else
      # configure && make & make install by default
      first_item="$(head -1 ${i_pkg_bn}.list)"
      if [ -d ${first_item} ]; then
        pushd ${first_item} > /dev/null
      else
        pushd $(dirname ${first_item}) > /dev/null
      fi
      
      ./configure && make && make install
      ecode="$?"
      popd > /dev/null
    fi
    if [ "${ecode}" != "0" ]; then
      lognl "....tar-ball compilation & installation failed (${i_pkg_bn}) Stop."
      let "result++"
      break
    else
      lognl "..tar-ball compilation & installation succeeded (${i_pkg_bn})"
    fi
    
  done
  
  return ${result}
}


# ###########################################################################
# install_ant ( [install_to_path] )
#   performs ant installation from binary package
#   if 'install_to_path' switch is given, PATH var. is going to be modified
#   otherwise ANT_EXE is set to ant binary
# ###########################################################################

function install_ant ()
{
  tool_name="ant"
  
  lognl "install_ant started - ant installation procedure"
  # check ant installation
  check_tool_install ${tool_name}
  if [ "$?" == "0" ]; then
    lognl ".${tool_name} found in $(which ${tool_name} | head -1)"
    lognl ".${tool_name} version $(${tool_name} -version)"
  else
    # getting last ant version from web
    ANT_SITE="http://apache.mirror.superhosting.cz/ant/binaries/"
    ANT_BINARY="$(curl ${ANT_SITE} 2> /dev/null | tr '>' '\n' | tr '<' '\n' | grep 'ant-' | grep 'bin' | grep 'tar.bz2$' | head -1)"
    
    if [ -z "${ANT_BINARY}" ]; then
      # no ant found error
      lognl ".ERROR:${tool_name} binary package not found on the web"
      return 1
    else
      lognl ".${tool_name} binary package found on the web"
    fi
    
    # downloading ant
    curl ${ANT_SITE}/${ANT_BINARY} > ${ANT_BINARY} 2> /dev/null
    if [ ! -e ${ANT_BINARY} ];then
      # download error
      lognl ".ERROR:${tool_name} binary package download failed (${ANT_BINARY})"
      return 1
    else
      lognl ".${tool_name} binary package downloaded (${ANT_BINARY})"
    fi
    
    # extracting ant
    tar xjf ${ANT_BINARY}
    ANT_BINDIR="$(dirname $(tar tjf ${ANT_BINARY} | grep 'bin/ant$'))"
    
    # test dir
    if [ ! -d ${ANT_BINDIR} -o ! -x ${ANT_BINDIR}/ant ]; then
      # extraction error
      lognl ".ERROR:${tool_name} extraction failed!"
      return 1 
    else
      lognl ".${tool_name} extracted (${ANT_BINARY} -> ${PWD}/${ANT_BINDIR})"
    fi
    
    # set ant variables & installation verification
    export ANT_HOME="${PWD}/${ANT_BINDIR}/.."
    export ANT_EXE="${PWD}/${ANT_BINDIR}/ant"
    
    if [ "$1" == "install_to_path" ]; then
      export PATH="${PATH}:${PWD}/${ANT_BINDIR}"
      check_tool_install ${tool_name}
      if [ "$?" == "0" ]; then
        lognl ".${tool_name} version $(which ${tool_name} | head -1) installed in \${PATH}"
      else
        lognl ".ERROR:${tool_name} cannot be launched! (final step problem)"
        return 1
      fi
    else
      # verify ant via ${ANT_EXE}
      ${ANT_EXE} -version
      if [ "$?" == "0" ]; then
        lognl ".${tool_name} version $(${ANT_EXE} -version) installed in ${ANT_EXE}, \${ANT_EXE} exported"
        return 0
      else
        # final run error
        lognl ".ERROR:${tool_name} cannot be launched! (final step problem)"
        return 1
      fi
    fi

  fi
}



# ###########################################################################
# install_ibm_java ( [rhel5 java version] [rhel4 java version] )
#   performs ibm java installation from rpm 
#   {java + java-devel + java-javacomm} 
#   by default java v1.5 for RHEL4 and v1.6 for RHEL5 are installed
#   ex. 'install_ibm_java 1.5.0 1.5.0'
# ###########################################################################

function install_ibm_java ()
{
    rhel5_ver="$1"
    rhel4_ver="$2"

    lognl "install_ibm_java started - ibm java installation procedure"
    
    # get os architecture
    arch="$(uname -i)"
    
    # get RHEL version
    main_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $1}')"
    minor_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $2}')"
    if [ -z "${minor_ver}" ]; then
        minor_ver="6"    # default minor version
    fi


    if [ "${main_ver}" == "4" ]; then
        # RHEL 4.x branch
        if [ ! -z "${rhel4_ver}" ]; then
            pkg_head="java-${rhel4_ver}-ibm-"
        elif [ ! -z "${rhel5_ver}" ]; then 
            # If no explicit RHEL4 version is given, use RHEL5 version as well
            pkg_head="java-${rhel5_ver}-ibm-"
        else
            pkg_head="java-1.5.0-ibm-"
        fi
        # query java packages
        rpm -q ${pkg_head}javacomm ${pkg_head}devel ${pkg_head}  >/dev/null 2>&1
        if [ "$?" == "0" ]; then
            lognl ".java packages found - skipping installation"
            return 0
        fi
        
        # upgrade libXp
        rpm -q libXp >/dev/null 2>&1
        if [ "$?" != "0" ]; then
            lognl ".installing libXp from xorg-x11-deprecated-libs packages"
            yum -y install xorg-x11-deprecated-libs
        fi
        
        # upgrade alsa-lib, compat-libstdc++-33, jpackage-utils
        install_packages_yum alsa-lib compat-libstdc++-33 jpackage-utils
        
        # download url
        down_url="http://porkchop.devel.redhat.com/released/RHEL-4-Extras/U${minor_ver}/AS/${arch}/tree/RedHat/RPMS"
        # checking number of files (needs to be more than 3)
        file_cnt="$(curl ${down_url}/ 2> /dev/null | tr '>' '\n' | tr '<' '\n' | grep "${pkg_head}" | grep "${arch}" | grep -c '.rpm$')"
        
        if [ "${file_cnt}" -lt "3" ]; then
            minor_ver="6" # switching to RHEL 4.6 data if not found
            down_url="http://porkchop.devel.redhat.com/released/RHEL-4-Extras/U${minor_ver}/AS/${arch}/tree/RedHat/RPMS"
        fi

        # downloading ibm java packages
        pkgs="${pkg_head}1. ${pkg_head}devel-1. ${pkg_head}javacomm-1."
        for i_pkg in ${pkgs}; do
            i_file="$(curl ${down_url}/ 2> /dev/null | tr '>' '\n' | tr '<' '\n' | grep "${i_pkg}" | grep "${arch}" | grep '.rpm$' | head -1)"
            
            lognl ".downloading ${i_file}"
            curl ${down_url}/${i_file} > ${i_file} 2> /dev/null
        done

        # ibm java rpm installation
        lognl ".ibm java installation"
        rpm -ivh ${pkg_head}*.rpm
        
        if [ "$?" != "0" ]; then
            lognl "..failed"
            return 1
        else
            lognl "..succeeded"
            return 0
        fi
        
    elif [ "${main_ver}" == "5" ]; then
        # RHEL 5.x branch
        if [ ! -z "${rhel5_ver}" ]; then
            pkg_head="java-${rhel5_ver}-ibm-"
        else
            pkg_head="java-1.6.0-ibm-"
        fi
        # query java packages
        rpm -q ${pkg_head}javacomm ${pkg_head}devel ${pkg_head} >/dev/null 2>&1
        if [ "$?" == "0" ]; then
            lognl ".java packages found"
            return 0
        fi
        
        # upgrade libXp
        rpm -q libXp >/dev/null 2>&1
        if [ "$?" != "0" ]; then
            lognl ".installing libXp from xorg-x11-deprecated-libs packages"
            yum -y install xorg-x11-deprecated-libs
        fi
        
        # upgrade alsa-lib, compat-libstdc++-33, jpackage-utils
        install_packages_yum alsa-lib compat-libstdc++-33 jpackage-utils
        
        # download url
        down_url="http://porkchop.devel.redhat.com/released/RHEL-5-Server-Supplementary/U${minor_ver}/${arch}/os/Supplementary"
        
        # checking number of files (needs to be more than 3)
        file_cnt="$(curl ${down_url}/ 2> /dev/null | tr '>' '\n' | tr '<' '\n' | grep "${pkg_head}" | grep "${arch}" | grep -c '.rpm$')"
        
        if [ "${file_cnt}" -lt "3" ]; then
            minor_ver="2" # switching to RHEL 5.2 data
            down_url="http://porkchop.devel.redhat.com/released/RHEL-5-Server-Supplementary/U${minor_ver}/${arch}/os/Supplementary"
        fi

        # downloading ibm java packages
        pkgs="${pkg_head}1. ${pkg_head}devel-1. ${pkg_head}javacomm-1."
        for i_pkg in ${pkgs}; do
            i_file="$(curl ${down_url}/ 2> /dev/null | tr '>' '\n' | tr '<' '\n' | grep "${i_pkg}" | grep "${arch}" | grep '.rpm$' | head -1)"
            lognl ".downloading ${i_file}"
            curl ${down_url}/${i_file} > ${i_file} 2> /dev/null
        done
        
        lognl ".ibm java installation"
        rpm -ivh ${pkg_head}*.rpm
        
        if [ "$?" != "0" ]; then
            lognl "..failed"
            return 1
        else
            lognl "..succeeded"
            return 0
        fi
    else
        # do not know how to build
        lognl ".ERROR:don't know how to install ibm java on this system! Stop."
        return 1
    fi
}


# ###########################################################################
# install_yum_repofiles ( )
#   creates yum repofiles to /etc/yum.repos.d/ if they are missing
# ###########################################################################
function install_yum_repofiles ( )
{
  
  result=0                         # return code

  # cleaning yum repos - there is yum bug link to not matching yum repos signature
  yum clean all

  # get os architecture
  arch="$(uname -i)"
  
  # get RHEL version
  main_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $1}')"
  minor_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $2}')"
  if [ -z "${minor_ver}" ]; then
    minor_ver="6"    # default minor version
  fi

  # query yum custom repo files and if we are on RHEL4 we might add new repos
  if [ ! -e /etc/yum.repos.d/rhts-custom.repo -a "${main_ver}" == "4" ];  then
    echo "[rhts-custom46]" >  /etc/yum.repos.d/rhts-custom-rhel4u6.repo
    echo "baseurl=http://qafiler.bos.redhat.com/redhat/released/RHEL-4/U6/AS/${arch}/tree/../rhts-${arch}" >> /etc/yum.repos.d/rhts-custom-rhel4u6.repo
    echo "enabled=1" >> /etc/yum.repos.d/rhts-custom-rhel4u6.repo
    echo "gpgcheck=0" >> /etc/yum.repos.d/rhts-custom-rhel4u6.repo
    
    echo "[rhts-custom47]" >  /etc/yum.repos.d/rhts-custom-rhel4u7.repo
    echo "baseurl=http://qafiler.bos.redhat.com/redhat/released/RHEL-4/U7/AS/${arch}/tree/../rhts-${arch}" >> /etc/yum.repos.d/rhts-custom-rhel4u7.repo
    echo "enabled=1" >> /etc/yum.repos.d/rhts-custom-rhel4u7.repo
    echo "gpgcheck=0" >> /etc/yum.repos.d/rhts-custom-rhel4u7.repo
    
  fi

  return 0
}



# default mrg repofile under yum repos path
MRG_REPOFILE_DEFAULT_FN="mrg-staging.repo"

# ###########################################################################
# create_yum_repofile( yum_repodata_path )
#   creates yum repofile and installs it under yum repo dir /etc/yum.repos.d
# ###########################################################################

function create_yum_repofile ()
{
  # write yum repo content fo repo file
  echo "[mrg-staging]" > /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN}
  echo "name=MRG packages from $1" >> /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN}
  echo "baseurl=$1/\$basearch" >> /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN}
  echo "enabled=1" >> /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN}
  echo "gpgcheck=0" >> /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN}
  
  if [ -r /etc/yum.repos.d/${MRG_REPOFILE_DEFAULT_FN} ]; then
    return 0
  else
    return 1
  fi
}


# ###########################################################################
# install_mrg_packages( pkg, ...)
#   installs MRG packages for RHEL4, RHEL5 and also additional ones
# ###########################################################################
function install_mrg_packages ()
{
  
  lognl "install_mrg_packages started - mrg packages installation"
  
  # install few packages needed by MRG but not part of delivery
  yum clean all
  pkgs="libaio boost db4 db4-utils"
  # optional packages - might be commented out
  pkgs="${pkgs} cppunit cppunit-doc cppunit-devel"
  pkgs="${pkgs} xerces-c xerces-c-devel xerces-c-doc xqilla xqilla-doc"
  install_packages_yum ${pkgs} 
  if [ "$?" != "0" ]; then
    lognl "..ERROR:installation of MRG dependencies failed! (${pkgs}) Stop."
    return 1
  fi
  
  # get os architecture
  arch="$(uname -i)"
  
  # get RHEL version
  main_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $1}')"
  minor_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $2}')"
  
  lognl ".RHEL${main_ver}.${minor_ver} detected"
  
  
  # detect whether repo or path given to $MRG_REPOFILE
  TEST_PARAM_RHTS_TEST_MRG_REPOFILE_IS_DIR="$(echo ${TEST_PARAM_RHTS_TEST_MRG_REPOFILE} | head -1 | awk '{if ($0 ~ /.repo$/){printf "0"}else{printf "1"}}')"
  case ${main_ver} in
    4)
        # test mrg repofile test env. parameter
        if [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "stable" ]; then
          # use latest stable build from RHN (default)
          MRG_REPOFILE="http://yum.engineering.redhat.com/rel-eng/repos/dist-4E-mrg"
          lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
          create_yum_repofile ${MRG_REPOFILE}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
            return 1
          fi
        elif [ -z "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" -o \
               "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "candidate" ]; then
          # use latest night build (candidate)
          MRG_REPOFILE="http://yum.engineering.redhat.com/rel-eng/repos/dist-4E-mrg-candidate"
          lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
          create_yum_repofile ${MRG_REPOFILE}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
            return 1
          fi
        elif [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "qpid-test" -o \
               "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "qpid-test.lab.bos.redhat.com" ]; then
          # qpid-test.lab.bos.redhat.com mrg-staging option
          MRG_REPOFILE="http://qpid-test.lab.bos.redhat.com/mrg-staging-rhel4/mrg-staging.repo"
          lognl ".MRG repofile: ${MRG_REPOFILE}"
          # download & install mrg yum repofile
          fetch_repofile ${MRG_REPOFILE} ${MRG_REPOFILE_DEFAULT_FN}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:download failed! Stop."
            return 1
          fi
        else
          # all other paths (selection from dir or from repo file)
          MRG_REPOFILE="${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}"
          if [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE_IS_DIR}" == "0" ]; then
            # installing from yum directory -> using create_yum_repofile
            lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
            # download & install mrg yum repofile
            create_yum_repofile ${MRG_REPOFILE}
            if [ "$?" != "0" ]; then
              lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
              return 1
            fi
          else
            # installing from yum repofile -> using fetch_repofile
            lognl ".MRG repofile: ${MRG_REPOFILE}"
            # download & install mrg yum repofile
            fetch_repofile ${MRG_REPOFILE} ${MRG_REPOFILE_DEFAULT_FN}
            if [ "$?" != "0" ]; then
              lognl "..ERROR:repofile download failed! Stop."
              return 1
            fi
          fi
        fi
        
        # installation phase
        lognl ".MRG yum install"
        pkgs="amqp rhm rhm-docs qpidc qpidd qpidc-perftest qpidc-devel python-qpid qpid-java-client qpid-java-common qpid-java qpidd-ssl qpidd-acl qpidd-xml  qpidd-cluster qmf qmf-devel qpidd-devel"
        pkgs="amqp rhm rhm-docs qpidc qpidd qpidc-perftest qpidc-devel python-qpid qpid-java-client qpid-java-common qpid-java qpidd-ssl qpidd-acl   qpidd-cluster qmf qmf-devel qpidd-devel"
        install_packages_yum ${pkgs} 
        if [ "$?" != "0" ]; then
          lognl "..ERROR:installation failed! (${pkgs}) Stop."
          return 1
        fi
        ;;
    5)
        
        # test mrg repofile test env. parameter
        if [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "stable" ]; then
          # use latest stable build from RHN (default)
          MRG_REPOFILE="http://yum.engineering.redhat.com/rel-eng/repos/dist-5E-mrg"
          lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
          create_yum_repofile ${MRG_REPOFILE}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
            return 1
          fi
        elif [ -z "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" -o \
               "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "candidate" ]; then
          # use latest night build (candidate)
          MRG_REPOFILE="http://yum.engineering.redhat.com/rel-eng/repos/dist-5E-mrg-candidate"
          lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
          create_yum_repofile ${MRG_REPOFILE}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
            return 1
          fi
        elif [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "qpid-test" -o \
               "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" == "qpid-test.lab.bos.redhat.com" ]; then
          # qpid-test.lab.bos.redhat.com mrg-staging option
          MRG_REPOFILE="http://qpid-test.lab.bos.redhat.com/mrg-staging/mrg-staging.repo"
          lognl ".MRG repofile: ${MRG_REPOFILE}"
          # download & install mrg yum repofile
          fetch_repofile ${MRG_REPOFILE} ${MRG_REPOFILE_DEFAULT_FN}
          if [ "$?" != "0" ]; then
            lognl "..ERROR:download failed! Stop."
            return 1
          fi
        else
          # all other paths (selection from dir or from repo file)
          MRG_REPOFILE="${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}"
          if [ "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE_IS_DIR}" == "0" ]; then
            # installing from yum directory -> using create_yum_repofile
            lognl ".MRG repofile: ${MRG_REPOFILE} [is directory]"
            # download & install mrg yum repofile
            create_yum_repofile ${MRG_REPOFILE}
            if [ "$?" != "0" ]; then
              lognl "..ERROR:Repofile installation failed! Stop. ($(basename ${MRG_REPOFILE}))"
              return 1
            fi
          else
            # installing from yum repofile -> using fetch_repofile
            lognl ".MRG repofile: ${MRG_REPOFILE}"
            # download & install mrg yum repofile
            fetch_repofile ${MRG_REPOFILE} ${MRG_REPOFILE_DEFAULT_FN}
            if [ "$?" != "0" ]; then
              lognl "..ERROR:repofile download failed! Stop."
              return 1
            fi
          fi
        fi
        
        # installation phase
        pkgs="amqp rhm rhm-docs qpidc qpidd qpidd-devel qpidc-perftest qpidc-devel python-qpid qpid-java-client qpid-java-common qpid-java qpidd-ssl qpidd-acl qpidd-xml qpidd-rdma qpidd-cluster qmf qmf-devel "
        install_packages_yum  ${pkgs}
        if [ "$?" != "0" ]; then
          lognl "..ERROR:installation (${pkgs}) failed! Stop."
          return 1
        fi
        ;;
    *)
        lognl "Do not know how to install MRG for this os"
        return 1
        ;;
  
  esac

  # additional mrg packages install
  if [ "$*" != "" ]; then
    install_packages_yum $*
    if [ "$?" != "0" ]; then
      lognl "..ERROR:additional pkgs installation failed! ($*) Stop."
      return 1
    fi
  fi
  
  return 0
} # function install_mrg_packages ()


# ###########################################################################
# compile_mrg( )
#   compiles and installs MRG/M from sources
#   parameters are stored in ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} var:
#   <qpid revision> <store revision> [qpid svn url] [store svn url]
#   use '-' or 'HEAD' for last revisions
# ###########################################################################
function compile_mrg ()
{
  
  lognl "compile_mrg started - mrg compile & installation"
  
  # install few packages needed by MRG but not part of delivery
  yum clean all
  pkgs="libaio libaio-devel boost boost-devel db4 db4-devel db4-utils subversion ruby openais openais-devel xqilla xqilla-devel xerces-c"
  install_packages_yum ${pkgs} 
  if [ "$?" != "0" ]; then
    lognl "..ERROR:installation of MRG pre-dependencies failed! (${pkgs}) Stop."
    return 1
  fi
  
  # get os architecture
  arch="$(uname -i)"
  
  # get RHEL version
  main_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $1}')"
  minor_ver="$(head -1 /etc/issue | sed 's/[^0-9]/ /g' | gawk '{print $2}')"
  
  # adding autotools for RHEL4.x
  if [ "${main_ver}" == "4" ]; then
    install_source_tarball \
      http://ftp.gnu.org/gnu/automake/automake-1.9.6.tar.bz2 \
      ftp://ftp.asir.org/pub/FreeBSD/distfiles/automake-1.9.6.tar.bz2 \
      http://ftp.gnu.org/gnu/libtool/libtool-1.5.26.tar.gz \
      ftp://ftp.asir.org/pub/FreeBSD/distfiles/libtool-1.5.26.tar.gz \
      http://pkg-config.freedesktop.org/releases/pkg-config-0.21.tar.gz \
      ftp://83.166.201.99/gentoo/distfiles/pkg-config-0.21.tar.gz \
      ftp://ftp.ps.pl/dsk0/ftp.freebsd.org/FreeBSD/ports/distfiles/gnome2/pkg-config-0.21.tar.gz \
      ftp://ftp.gwdg.de/pub/FreeBSD/ports/distfiles/gnome2/pkg-config-0.21.tar.gz

    if [ "$?" != "0" ]; then
      lognl "..ERROR:installation of RHEL4.x autotools failed! (${pkgs}) Stop."
      return 1
    fi
  fi
  
  # overcome buggy behavior of bash 3.00.15(1)
  if [ "${main_ver}" == "4" ]; then
    # PATH update to force rehash !
    local_path=${PATH}
    export PATH="/usr/local/bin:/usr/bin"
    export PATH=${local_path}
  fi
  
  # testing boost version
  boost_correct_ver="$(rpm -q boost | uniq | grep -c 'boost-1.3[3-9].')"
  lognl ".boost library version test"
  # boost need to be reinstalled
  if [ "${boost_correct_ver}" != "1" ]; then
    lognl "..boost needs to be reinstalled (remove & compile)"
    
    # remove current boost
    rpm -e boost boost-devel --allmatches >/dev/null 2>&1
    
    # query boost package
    rpm -q boost boost-devel >/dev/null 2>&1
    if [ "$?" != "2" ]; then
      lognl "..WARNING: rpm -e boost boost-devel --allmatches seems to failed...once more"
      rpm -e boost boost-devel --allmatches >/dev/null 2>&1
    fi
  
    # download boost v 1.35
    boost_dir="boost_1_35_0"
    boost_fn="${boost_dir}.tar.bz2"
    boost_url_list="http://ovh.dl.sourceforge.net/sourceforge/boost/${boost_fn} http://mesh.dl.sourceforge.net/sourceforge/boost/${boost_fn} http://dfn.dl.sourceforge.net/sourceforge/boost/${boost_fn} "
    
    for i_file in ${boost_url_list}; do
      # download boost
      wget ${i_file} >/dev/null 2>&1
      # test downloaded file
      tar tjf ${boost_fn} >/dev/null 2>&1
      ecode="$?"
      if [ "${ecode}" == "0" ]; then
        break
      else
        rm -f ${boost_fn}
      fi
    done 

    # final download test
    if [ "${ecode}" != "0" ]; then
      lognl "..ERROR: boost download failed! Stop."
      return 1
    fi
    
    # boost extraction
    tar xjf ${boost_fn}
    # test extraction
    if [ ! -d ${boost_dir} -o "$?" != "0"  ]; then
      lognl "..ERROR: boost extraction failed! Stop."
      return 1
    fi
    
    # boost compilation
    cd ${boost_dir}
    ./configure --prefix=/usr && make && make install
    if [ "$?" != "0"  ]; then
      lognl "..ERROR: boost compilation failed! Stop."
      cd ..
      return 1
    fi
    cd ..
  
    # boost customization
    if [ ! -d /usr/include/boost ]; then
      ln -s /usr/include/boost-1_35/boost /usr/include/boost
    fi
    
    # linking dynamic libraries
    lib_names="libboost_thread libboost_wserialization libboost_wave libboost_unit_test_framework libboost_system libboost_serialization libboost_regex libboost_python libboost_program_options libboost_prg_exec_monitor libboost_iostreams libboost_graph libboost_filesystem libboost_date_time"
    suffix="-gcc34-mt"
#  ln -s /usr/lib/libboost_thread-gcc34-mt.so /usr/lib/libboost_thread.so
    for i_lib in ${lib_names}; do
      if [ ! -r /usr/lib/${i_lib}.so -a -r /usr/lib/${i_lib}${suffix}.so ]; then
        lognl "linking /usr/lib/${i_lib}.so"
        ln -s /usr/lib/${i_lib}${suffix}.so /usr/lib/${i_lib}.so
      fi
    done
    
    # check of the boost_thread in /usr/lib even for x86_64
    if [ "${arch}" == "x86_64" -a ! -r /usr/lib64/libboost_thread-gcc34-mt-1_35.so.1.35.0 ]; then
      ln -s /usr/lib/libboost_thread-gcc34-mt-1_35.so.1.35.0 /usr/lib64/libboost_thread-gcc34-mt-1_35.so.1.35.0
    fi 
    
    # removing extra semicolons (for gcc 3.4.x)
    src_path="/usr/include/boost/ptr_container"
    src_list="ptr_deque.hpp ptr_list.hpp ptr_vector.hpp"
    
    for i_file in ${src_list}; do
      # create backup
      if [ -r ${src_path}/${i_file} -a \
         ! -r ${src_path}/${i_file}.00 ]; then
        cat ${src_path}/${i_file} > ${src_path}/${i_file}.00
      fi
      
      # modify code - remove semicolon
      gawk 'BEGIN{f=0}{if($0 ~ /BOOST_PTR_CONTAINER_DEFINE_SEQEUENCE_MEMBERS/){f=1} if((f==1)&&($0 ~ /);/)){gsub(/);/,")");f=0} print}' ${src_path}/${i_file}.00 > ${src_path}/${i_file}
    done
    
    export RHTS_TEST_MRG_BOOST_COMPILED="1.35.0"
    
    lognl "..boost compilation succeeded"
  else
    lognl "..boost version is OK, no reinstallation needed"
  fi
  
  
  # get checkout parameters
  if [ -n "$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $3}')" ]; then
    qpid_svn_url="$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $3}')"
  else
    qpid_svn_url="http://svn.apache.org/repos/asf/incubator/qpid/trunk/qpid"
  fi
  if [ -n "$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $4}')" ]; then
    store_svn_url="$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $4}')"
  else
    store_svn_url="http://anonsvn.jboss.org/repos/rhmessaging/store/trunk"
  fi
  
  # checkout journal store sources
  co_pwd="${PWD}"
  store_rev="$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $2}')"
  lognl ".qpid journal store sources checkout started... (${store_svn_url}:${store_rev})"
  if [ "${store_rev}" != "-" -a "${store_rev}" != "HEAD" ]; then
    svn co ${store_rev} ${store_svn_url} store > /dev/null
    ecode="$?"
  else
    svn co ${store_svn_url} store > /dev/null
    ecode="$?"
  fi
  # check checkout status
  if [ "${ecode}" != "0" ]; then
    lognl "..ERROR:qpid journal store sources checkout failed! Stop."
    return 1
  fi
  
  # checkout the qpid sources
  qpid_rev="$(echo ${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS} | awk '{print $1}')"
  lognl ".main qpid sources checkout started... (${qpid_svn_url}:${qpid_rev})"
  if [ "${qpid_rev}" != "-" -a "${qpid_rev}" != "HEAD" ]; then
    svn co ${qpid_rev} ${qpid_svn_url} qpid > /dev/null
    ecode="$?"
  else
    svn co ${qpid_svn_url} qpid > /dev/null
    ecode="$?"
  fi
  # check checkout status
  if [ "${ecode}" != "0" ]; then
    lognl "..ERROR:main qpid sources checkout failed! Stop."
    return 1
  fi
  
  # installation directory
  inst_dir="${HOME}/mrg_installed"
  export RHTS_TEST_MRG_COMPILED_DIR="${inst_dir}"
  
  # create directory
  trasher ${inst_dir}
  mkdir -p ${inst_dir}
  
  # compiling qpid sources
  if [ -x qpid/cpp/bootstrap -a -r qpid/cpp/bootstrap ]; then
    pushd qpid/cpp > /dev/null
    ./bootstrap && ./configure --prefix=${inst_dir} && make
    if [ "$?" != "0" ]; then
      lognl "..ERROR:main qpid sources compilation failed! Stop."
      popd > /dev/null
      return 1
    fi
    make check
    if [ "$?" != "0" ]; then
      lognl "..WARNING:main qpid sources check compile failed!"
    fi
    make install
    if [ "$?" != "0" ]; then
      lognl "..ERROR:main qpid sources installation failed! Stop."
      popd > /dev/null
      return 1
    fi
    
    # copy testing tools
    cp src/tests/.libs/* ${inst_dir}/bin
    
    cp src/tests/ssl_test src/tests/fanout_perftest \
       src/tests/federated_topic_test src/tests/multiq_perftest \
       src/tests/perfdist src/tests/federation.py \
       src/tests/header_test.py src/tests/restart_cluster \
       src/tests/vg_check src/tests/acl.py src/tests/ais_check \
       src/tests/benchmark src/tests/shared_perftest ${inst_dir}/bin
    cp src/tests/start_* src/tests/stop_* ${inst_dir}/bin
    cp src/tests/topic* src/tests/run* ${inst_dir}/bin
    cp src/tests/quick* ${inst_dir}/bin
    
    popd > /dev/null
  else
    lognl "..ERROR:main qpid sources: bootstrapping code not found! Stop."
    return 1
  fi
  
  
  # compiling journal store sources
  if [ -x store/cpp/bootstrap -a -r store/cpp/bootstrap ]; then
    pushd store/cpp > /dev/null
    ./bootstrap 
    if [ "$?" != "0" ]; then
      lognl "..ERROR:qpid journal sources boostrapping failed! Stop."
      popd > /dev/null
      return 1
    fi
    
    # patching store sources on RHEL4.x
    if [ "${main_ver}" == "4" -a -d rhel4-support ]; then
      pushd rhel4-support >/dev/null
      make apply
      if [ "$?" != "0" ]; then
        lognl "..ERROR:qpid journal sources RHEL4 sources patch failed! Stop."
        popd > /dev/null
        return 1
      fi
      popd > /dev/null
    fi
    
    ./configure --prefix=${inst_dir} --with-qpid-checkout=${co_pwd}/qpid && make
    if [ "$?" != "0" ]; then
      lognl "..ERROR:qpid journal sources compilation failed! Stop."
      popd > /dev/null
      return 1
    fi
    make check
    if [ "$?" != "0" ]; then
      lognl "..WARNING:qpid journal sources check compile failed!"
    fi
    make install
    if [ "$?" != "0" ]; then
      lognl "..ERROR:qpid journal sources installation failed! Stop."
      popd > /dev/null
      return 1
    fi
    popd > /dev/null
  else
    lognl "..ERROR:qpid journal sources: bootstrapping code not found! Stop."
    return 1
  fi
  
  # copy amqp specification
  if [ -d qpid/specs ]; then
    mkdir ${RHTS_TEST_MRG_COMPILED_DIR}/share/amqp
    cp qpid/specs/* ${RHTS_TEST_MRG_COMPILED_DIR}/share/amqp/
    TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_AMQP_SPEC=${RHTS_TEST_MRG_COMPILED_DIR}/share/amqp
    mkdir -p ${RHTS_TEST_MRG_COMPILED_DIR}/lib/python
    echo "amqp_spec = \"${RHTS_TEST_MRG_COMPILED_DIR}/share/amqp/amqp.0-10-qpid-errata.xml\"" > ${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid_config.py
    echo "" >> ${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid_config.py
  else
    lognl "..WARNING:amqp specification not found! Python code might need it!"
  fi
  # compiling and installing python code
  if [ -e qpid/python/setup.py -a \
       -d qpid/python/commands -a \
       -d qpid/python/mllib ]; then
    pushd qpid/python > /dev/null
    # install qpid packages
    python setup.py install --home=${RHTS_TEST_MRG_COMPILED_DIR}
    
    # install mllib packages
    cp -R mllib ${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid/
    
    # install tests and docs packages
    cp -R tests* doc ${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid/
    
    # install python qpid tools
    cp commands/* ${RHTS_TEST_MRG_COMPILED_DIR}/bin/
    
    popd > /dev/null
    
    # add paths to PYTHONPATH
    if [ -z "${PYTHONPATH}" ]; then
      export TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_PYTHONPATH=${RHTS_TEST_MRG_COMPILED_DIR}/lib/python:${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid
    else
      export TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_PYTHONPATH=${RHTS_TEST_MRG_COMPILED_DIR}/lib/python:${RHTS_TEST_MRG_COMPILED_DIR}/lib/python/qpid:${PYTHONPATH}
    fi
  
  else
    lognl "..WARNING:qpid python sources not found! Python code installation skipped."
  fi
  pushd qpid/cpp > /dev/null
  
  
  # add proper directories to the PATH (store it into the config file)
  export TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_PATH=${inst_dir}/bin:${inst_dir}/sbin:${PATH}
  
  if [ -z "${LD_LIBRARY_PATH}" ]; then
    export TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_LD_LIBRARY_PATH=${inst_dir}/lib:${inst_dir}/qpid/daemon:/usr/local/lib:/usr/local/lib64:/usr/lib:/usr/lib64:/lib:/lib64
  else
    export TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_LD_LIBRARY_PATH=${inst_dir}/lib:${inst_dir}/qpid/daemon:${LD_LIBRARY_PATH}
  fi
  
  # make note on gcc/g++ include and library paths
  export RHTS_TEST_MRG_CXXFLAGS="-I${inst_dir}/include"
  export RHTS_TEST_MRG_CFLAGS="-I${inst_dir}/include"
  export RHTS_TEST_MRG_LFLAGS="-L${inst_dir}/lib"
  
  return 0
} # function compile_mrg ()


# ###########################################################################
# uninstall_mrg( [all_pkgs] [just_main_pkgs] 
#                [delete_mrg_compiled] [hide_mrg_compiled] )
#   packages: [all_pkgs] removes MRG and also dependencies
#             [just_main_pkgs] removes just MRG (default)
#   MRG src compilation: [delete_mrg_compiled] deletes ~/mrg_compiled if exists
#                        [hide_mrg_compiled] moves ~/mrg_compiled if exists
#   uninstalls MRG packages and also hides mrg compiled (uses yum)
# ###########################################################################
function uninstall_mrg ()
{
  # parse arguments
  if [ -z "$*" ]; then
    args="just_main_pkgs hide_mrg_compiled"
  else
    args="$*"
  fi
  
  # package part
  main_pkgs="python-qpid qpid-java-client qpid-java-common qpidc qpidc-devel qpidc-debuginfo qpidc-perftest qpidd qpidd-devel rhm rhm-docs rhm-debuginfo  qpid-java qpidd-ssl qpidd-acl qpidd-xml qpidd-rdma qpidd-cluster cumin qmf qmf-devel"
  other_pkgs="python-psycopg2 tuna xerces-c xerces-c-doc xerces-c-devel xerces-c-debuginfo xqilla xqilla-devel xqilla-debuginfo"
  if [ "$(echo ${args} | grep -c 'all_pkgs')" == "1" ]; then
    yum -y remove ${main_pkgs} ${other_pkgs}
    #rpm -e ${main_pkgs} ${other_pkgs} --allmatches
  elif [ "$(echo ${args} | grep -c 'just_main_pkgs')" == "1" ]; then
    yum -y remove ${main_pkgs}
    #rpm -e ${main_pkgs} --allmatches
  fi
  
  # compilation part
  if [ "$(echo ${args} | grep -c 'hide_mrg_compiled')" == "1" ]; then
    if [ -n "${RHTS_TEST_MRG_COMPILED_DIR}" -a \
         -d "${RHTS_TEST_MRG_COMPILED_DIR}" ]; then
      mv ${RHTS_TEST_MRG_COMPILED_DIR} \
        "${RHTS_TEST_MRG_COMPILED_DIR}_$(date +%Y%m%d_%H%M%S)"
    fi
  elif [ "$(echo ${args} | grep -c 'delete_mrg_compiled')" == "1" ]; then
    if [ -n "${RHTS_TEST_MRG_COMPILED_DIR}" ]; then
      trasher ${RHTS_TEST_MRG_COMPILED_DIR}
    fi
  fi
  
  return 0
}


# ###########################################################################
# mrg_src_checkout( [revs_urls] [dirs] )
#   revs_urls - mrg checkout revisions and locations: (2/4 fields)
#               <qpid revision> <store revision> [qpid svn url] [store svn url]                 ex.  -r72344          -r2032 
#          shortcuts for trunk are "- -" and "HEAD HEAD"
#          TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS can be used for revs_urls
#   dirs - mrg checkout directories, syntax: (has to be 2 fields)
#          <qpid_dir> <store_dir>
#
#   checks-out the mrg sources of revisions revs from location urls to 
#   directories dirs
# ###########################################################################
function mrg_src_checkout ()
{
  ret_code=0
  checkout_basedir="mrg_checkout"
  qpid_rev=""
  store_rev=""
  qpid_svn_url="http://svn.apache.org/repos/asf/incubator/qpid/trunk/qpid"
  store_svn_url="http://anonsvn.jboss.org/repos/rhmessaging/store/trunk"
  qpid_dir="qpid-trunk"
  store_dir="store-trunk"

  # revision and url check
  i_cnt=0
  for i_item in $1; do
    let "i_cnt++"
    
    case ${i_cnt} in
      1)
         qpid_rev="${i_item}"
         ;;
      2)
         source_rev="${i_item}"
         ;;
      3)
         qpid_svn_url="${i_item}"
         ;;
      4)
         store_svn_url="${i_item}"
         ;;
      *)
         lognl "mrg_src_checkout:WARNING:first parameter syntax issue ($1)"
         ;;
    esac
  done
  
  # directory check and assignment of dest dirs
  if [ -n "$2" ]; then
    qpid_dir="$(echo $2 | awk '{print $1}')"
    store_dir="$(echo $2 | awk '{print $2}')"
  else
    # adding revision to directories
    if [ "${store_rev}" != "-" -a "${store_rev}" != "HEAD" ]; then
      store_dir="${store_dir}${store_rev}"
    fi
    if [ "${qpid_rev}" != "-" -a "${qpid_rev}" != "HEAD" ]; then
      qpid_dir="${qpid_dir}${qpid_rev}"
    fi
  fi
  
  # creation co base directory
  if [ ! -d ${checkout_basedir} ]; then
    mkdir -p ${checkout_basedir}
  fi
  
  # checkout journal store sources
  co_pwd="${PWD}"
  pushd ${checkout_basedir}  > /dev/null
  trasher ${store_dir}
  lognl ".qpid journal store sources checkout started... (${store_svn_url}:${store_rev})"
  if [ "${store_rev}" != "-" -a "${store_rev}" != "HEAD" ]; then
    svn co ${store_rev} ${store_svn_url} ${store_dir} > /dev/null
    ecode="$?"
  else
    svn co ${store_svn_url} ${store_dir} > /dev/null
    ecode="$?"
  fi
  popd > /dev/null
  
  # check checkout status
  if [ "${ecode}" != "0" ]; then
    lognl "mrg_src_checkout:ERROR:qpid journal store sources checkout failed!"
    let "ret_code++"
  fi
  
  # checkout the qpid sources
  lognl ".main qpid sources checkout started... (${qpid_svn_url}:${qpid_rev})"
  pushd ${checkout_basedir}  > /dev/null
  trasher ${store_dir}
  if [ "${qpid_rev}" != "-" -a "${qpid_rev}" != "HEAD" ]; then
    svn co ${qpid_rev} ${qpid_svn_url} ${qpid_dir} > /dev/null
    ecode="$?"
  else
    svn co ${qpid_svn_url} ${qpid_dir} > /dev/null
    ecode="$?"
  fi
  popd > /dev/null
  # check checkout status
  if [ "${ecode}" != "0" ]; then
    lognl "mrg_src_checkout:ERROR:main qpid sources checkout failed!"
    let "ret_code++"
  fi
  
  # export the directories
  export RHTS_TEST_MRG_SRC_CHECKOUT_STORE_DIR="${PWD}/${checkout_basedir}/${store_dir}"
  export RHTS_TEST_MRG_SRC_CHECKOUT_QPID_DIR="${PWD}/${checkout_basedir}/${qpid_dir}"
  
  # verbose end message
  if [ "${ret_code}" == "0" ]; then
    lognl "mrg_src_checkout:finished succesfully to dirs (${checkout_basedir}/${store_dir};${checkout_basedir}/${qpid_dir})"
  fi
  
  return ${ret_code}
}

function mrg_src_co ()
{
  mrg_src_checkout $*
  return $?
}


# ###########################################################################
# ###########################################################################
#                   MRG broker process management functions
# ###########################################################################
# ###########################################################################

# ###########################################################################
# qpidd_start( port, data-dir, log-transcript, other-options )
#   
#   starts one qpidd instance as daemon
# ###########################################################################
function qpidd_start_daemon ()
{
  qpidd_start "daemon" $*
  return $?
}


# ###########################################################################
# qpidd_start( port, data-dir, log-transcript, other-options )
#   
#   starts one qpidd instance as normal program on background
# ###########################################################################
function qpidd_start_normal ()
{
  qpidd_start "normal" $*
  return $?
}



# ###########################################################################
# qpidd_start( type, port, data-dir, log-transcript, other-options )
#   
#   starts one qpidd instance
# ###########################################################################
function qpidd_start ()
{
  # default parameters
  wait_after_qpidd_launch=1
  qpidd_def_port="5672"
  if [ -z "${DATE_TIME_START}" ]; then
    export DATE_TIME_START=$(date +%Y%m%d_%H%M%S)
  fi
  qpidd_def_datadir="${PWD}/rhts_qpidd/${DATE_TIME_START}"
  qpidd_def_transcript="${MRG_CPP_BROKER}.log"
  qpidd_inst_cnt="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | wc -l )"
  
  # detect qpidd wrapper
  if [ -z "${VALGRIND_WRAPPER}" ]; then
    qpidd_wrapper=""
  else
    qpidd_wrapper="${VALGRIND_WRAPPER}"
  fi
  
  if [ -n "${QPIDD_LOCK}" ]; then
    qpidd_lock="${QPIDD_LOCK}"
  else
    qpidd_lock="${MRG_CPP_BROKER}.${qpidd_inst_cnt}.lock"
  fi
  
  if [ -n "${QPIDD_ECODE}" ]; then
    qpidd_ecode="${QPIDD_ECODE}"
  else
    qpidd_ecode="${MRG_CPP_BROKER}.${qpidd_inst_cnt}.ecode"
  fi
  
  # run type branching
  if [ "$1"  == "service" ]; then
    # starting qpidd as service daemon
    qpidd_start_service
    return $?
  else
    # common custom qpidd launch part goes below... [normal/daemon]
    
    # port definition
    if [ -z "$2" -o "$2" == "-" ]; then
      # second parameter not defined
      if [ -n "${QPIDD_PORT}" -a \
           "$(echo ${QPIDD_PORT} | egrep -c '^[0-9]+$')" == "1" ]; then
        port="${QPIDD_PORT}"
      else
        port="${qpidd_def_port}"
      fi
    else
      # port defined trying to use
      if [ "$(echo $2 | egrep -c '^[0-9]+$')" == "1" ]; then
        port="$2"
      else
        port="${qpidd_def_port}"
      fi
    fi
    
    # data-dir definition - checking validity
    if [ -z "$3" -o "$3" == "-" -o ! -d $3 ]; then
      # parameter not defined, verify QPIDD_DATADIR*
      if [ -n "${QPIDD_DATADIR}" -a \
           -n "${QPIDD_DATADIR_JOURNAL}" -a \
           -d "${QPIDD_DATADIR_JOURNAL}" -a \
           -w "${QPIDD_DATADIR_JOURNAL}" ]; then
        
        if [ "$(ls -lA ${QPIDD_DATADIR_JOURNAL} | egrep -vc '^total [0-9]+')" != "0" -a \
             "$(echo ${QPIDD_DATADIR_JOURNAL} | tr '/' ' ' | wc -w)" -ge "2" ]; then
          rm -rf ${QPIDD_DATADIR_JOURNAL}/*
        fi
        datadir="${QPIDD_DATADIR_JOURNAL}"
      else
        trasher ${qpidd_def_datadir}
        mkdir -p ${qpidd_def_datadir}
        qpidd_def_datadir_journal="$(mktemp -d ${qpidd_def_datadir}/${MRG_CPP_BROKER}.XXXX)"
        datadir="${qpidd_def_datadir_journal}"
        
        export QPIDD_DATADIR_JOURNAL="${datadir}"
        export QPIDD_DATADIR="$(dirname ${datadir})"
      fi
    else
      # parameter defined using it...
      if [ -d  "$3" -a -w "$3" ]; then
        datadir="$3"
      else
        trasher ${qpidd_def_datadir}
        mkdir -p ${qpidd_def_datadir}
        qpidd_def_datadir_journal="$(mktemp -d ${qpidd_def_datadir}/${MRG_CPP_BROKER}.XXXX)"
        datadir="${qpidd_def_datadir_journal}"
        
        export QPIDD_DATADIR_JOURNAL="${datadir}"
        export QPIDD_DATADIR="$(dirname ${datadir})"
      fi
      
    fi
    
    # transcript-log definition
    if [ -z "$4" -o "$4" == "-" -o -d "$4" ]; then
      # parameter not defined
      transcript=${qpidd_def_transcript}
    else
      # parameter defined
      trasher $4
      transcript="$4"
    fi
    
    # other params
    if [ -z "$5" -o "$5" == "-" ]; then
      test_params="${QPIDD_TEST_PARAMS}"
    else
      i_cnt=1
      test_params=""
      
      # append function command line params
      for i_arg in $*; do
        if [ "${i_cnt}" -gt "4" ]; then
          test_params="${test_params} ${i_arg}"
        fi
        let "i_cnt++"
      done
    
    fi
    
    # main qpidd launch
    if [ "$1"  == "daemon" ]; then
      # starting qpidd as daemon
      $( touch ${qpidd_lock} ; ${qpidd_wrapper} ${QPIDD_DIR}/${MRG_CPP_BROKER} --daemon --data-dir ${datadir} --port ${port} ${test_params} > ${transcript} 2> ${transcript}; echo $? > ${qpidd_ecode} ; rm -f ${qpidd_lock} )
    
      lognl "qpidd_start: qpidd launched daemon way (port:${port}, log:${transcript}, params:${test_params})"
    else
      # starting qpidd as normal application on background
      $( touch ${qpidd_lock} ; ${qpidd_wrapper} ${QPIDD_DIR}/${MRG_CPP_BROKER} --data-dir ${datadir} --port ${port} ${test_params} > ${transcript} 2>${transcript}; echo $? > ${qpidd_ecode} ; rm -f ${qpidd_lock} ) &
      
      lognl "qpidd_start: qpidd launched normal bg way (port:${port}, log:${transcript}, params:${test_params})"
    fi
  fi
  
  
  # final variable export
  export QPIDD_PORT="${port}"
  export QPIDD_TEST_PARAMS="${test_params}"
  export QPIDD_TRANSCRIPT="${transcript}"
  
  sleep ${wait_after_qpidd_launch}
  
  # get fresh qpidd status (be quiet)
  qpidd_status quiet
  
  return 0
}


# ###########################################################################
# qpidd_start_service(  )
#   
#   starts one qpidd as the service using 'service qpidd start'
# ###########################################################################
function qpidd_start_service ()
{
  # start qpidd service
  service ${MRG_CPP_BROKER} start
  ecode="$?"
  # get fresh qpidd status (be quiet)
  qpidd_status quiet
  lognl "qpidd_start: qpidd launched service-daemon way"
  return "${ecode}"
}




# ###########################################################################
# qpidd_stop( [PID] )
#   
#   PID - is particular qpidd PID (optional parameter)
#   stops one (PID param given) or all qpidd instances
# ###########################################################################
function qpidd_stop ()
{
  wait_after_kill="5"
  if [ -z "$1" ]; then
    # no argument - quitting all qpidd instances
    pids="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | awk '{printf $1 " "}' )"
    if [ -z "${pids}" ]; then
      # no argument - quitting all qpidd instances - no qpidd found
      lognl "qpidd_stop: No qpidd broker found to stop!"
      export QPIDD_INST_TYPE=""
      export QPIDD_INST_PID=""
      export QPIDD_INST_PORT=""
      export QPIDD_INST_CNT="0"
      return 0
    else
      # no argument - quitting all qpidd instances - a qpidd instance found
      lognl "qpidd_stop: following qpidd brokers are found to stop (${pids})"
      kill_process_id ${pids} 2 quiet
      ecode="$?"
      sleep ${wait_after_kill}
      pids="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | awk '{printf $1 " "}' )"
      if [ -n "${pids}" ]; then
        kill_process_id ${pids} 9 quiet
        ecode="$?"
      fi
      qpidd_status quiet
      return ${ecode}
    fi
  else
    # pid argument
    pid_check="$(ps -aeo pid,args --no-headers | grep ${MRG_CPP_BROKER} | grep -c "^[ ]*$1 ")"
    if [ "${pid_check}" == "1" ]; then
      # pid argument - qpidd instance of specified PID found
      lognl "qpidd_stop: qpidd broker (pid:$1) found to stop"
      kill_process_id $1 2
      ecode="$?"
      sleep ${wait_after_kill}
      pid_check="$(ps -aeo pid,args --no-headers | grep ${MRG_CPP_BROKER} | grep -c "^[ ]*$1 ")"
      if [ "${pid_check}" == "1" ]; then
        kill_process_id $1 9
        ecode="$?"
      fi
      qpidd_status quiet
      return ${ecode}
    else
      # pid argument - qpidd instance of specified PID not found!
      lognl "qpidd_stop: No qpidd broker (pid:$1) found to stop!"
      qpidd_status quiet
      return 0
    fi
  fi
}



# ###########################################################################
# qpidd_status( [quiet])
#   
#   gets status of all qpidd instances
#   stores it to :
#     QPIDD_INST_TYPE qpidd instance type (N/D/S) ~ normal/daemon/service-daemon
#     QPIDD_INST_PID  instance pid[s]
#     QPIDD_INST_PORT instance port[s]
#     QPIDD_INST_CNT  instance count
# ###########################################################################
function qpidd_status ()
{
  
  # get qpidd pids
  pids="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | awk '{printf $1 " "}' )"
  pid_cnt="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | wc -l )"

  port=""
  type=""
  for i_pid in ${pids}; do
    # getting the port where is qpidd listening
    prt="$(netstat -ntlp | grep "${i_pid}/${MRG_CPP_BROKER}" | awk '{split($4,arr,/:/);print arr[2]}')"
    if [ -z "${prt}" ]; then
      port="${port} -"
    else
      port="${port} ${prt}"
    fi
    
    # getting qpidd instance type
    daemon_flag="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | grep "^[ ]+${i_pid}" | egrep '(-d|--daemon)' | wc -l)"
    daemon_var_flag="$(ps -aeo pid,args --no-headers | awk '{if ($2~/'"${MRG_CPP_BROKER}"'/){print}}' | grep "^[ ]+${i_pid}"| egrep '(-d|--daemon)' | grep '/var/lib/qpidd' | wc -l)"
    
    if [ "${daemon_flag}" == "1" ]; then
      if [ "${daemon_var_flag}" == "1" ]; then
        type="${type} S"
      else
        type="${type} D"
      fi
    else
      type="${type} N"
    fi
    
  done
  
  export QPIDD_INST_TYPE="${type}"
  export QPIDD_INST_PID="${pids}"
  export QPIDD_INST_PORT="${port}"
  export QPIDD_INST_CNT="${pid_cnt}"

  if [ "$1" != "quiet" ]; then
    lognl "qpidd_status: ${pid_cnt} instance[s] running (pids:${pids}, ports:${port})"
  fi

}

# ###########################################################################
# qpidd_wait_on_settle( qpidd_transcript_log
#                       text_to_be_searched4 or "-" [Listening on TCP port]
#                       qpidd_lock or "-"
#                       timeout=10   )
#   
#   waits for qpidd settlement
# ###########################################################################
function qpidd_wait_on_settle ()
{
  result=0                         # return code
  
  if [ -z "$1" -o -z "$2" -o -z "$3" ]; then
    lognl "qpidd_wait_on_settle: Bad or none parameters!"
    result=1
  else
    # get current timestamp
    c_ts=${SECONDS}
      
    # assign timeout
    if [ -z "$4" ]; then
      c_to="10"
    else
      c_to="$4"
    fi
    
    if [ "$2" == "-" ]; then
      search_phrase="Listening on TCP port"
    else
      search_phrase="$2"
    fi
    while [ 1 ]; do
      # file check & lock check
      if [ "$3" == "-" -a -r "$1" ]; then
        # no not apply qpidd lock file check
        if [ "$(grep -c "${search_phrase}" $1 2> /dev/null)" != "0" ]; then
          result=0
          break
        fi
      else
        # apply qpidd lock file check
        if [ "$(grep -c "${search_phrase}" $1 2> /dev/null)" != "0" -a \
             -e "$3" ]; then
          result=0
          break
        fi
      fi
      
      # timeout check
      if [ "$((${SECONDS}-${c_ts}))" -gt ${c_to} ]; then
        result=1
        break
      fi
      
    done
    
    if [ "${result}" == "0" ]; then
      lognl "qpidd_wait_on_settle: ${MRG_CPP_BROKER} started-up (dur:$((${SECONDS}-${c_ts}))sec)"
    else
      lognl "qpidd_wait_on_settle:ERROR:${MRG_CPP_BROKER} didn't started-up (dur:$((${SECONDS}-${c_ts}))sec)"
    fi
    
    
  fi
  
  return ${result}
}

# ###########################################################################
# ###########################################################################
#                      Process management functions
# ###########################################################################
# ###########################################################################


# ###########################################################################
# get_process_id ( process_name [,quiet] )
#   performs process search and returns found PID in env. variable FOUND_PID
#   FOUND_PID is either "" if nothing found or a number
# ###########################################################################
function get_process_id ()
{
  # default result variable
  FOUND_PID=""
  
  #assign kill_process_name arguments
  if [ -z "$1" ]; then
    lognl "get_process_id: process name not specified. Stop."
    return 1
  else
    f_process_name=$1
  fi

  if [ -z "$2" ]; then
    
    f_msgs=1
  else
    if [ "$2" == "quiet" ]; then
      f_msgs=0
    else
      f_msgs=1
    fi
  fi

  # search the process
  f_pid="`ps -aeo pid,args --no-headers | grep ${f_process_name} | grep -v 'grep' | gawk '{printf $1 " "}'`"

  if [ -z "${f_pid}" -a "${f_msgs}" == "1" ]; then
    lognl "get_process_id: process id not found. No such a process (${f_pid}) running. Stop."
    return 1
  else
    FOUND_PID=${f_pid}
  fi
  
  export FOUND_PID
  return 0
}

# ###########################################################################
# kill_process_name ( process_name [, kill signal [, verbose|quiet]] )
#   performs process kill
#   ex. kill_process_name xterm 7 -> kill -7 67676 where 67676 is pid of xterm
# ###########################################################################
function kill_process_name ()
{
  #assign kill_process_name arguments
  if [ -z "$1" ]; then
    lognl "kill_process_name: process name not specified. Stop."
    return 1
  else
    f_process_name=$1
  fi

  if [ -z "$2" ]; then
    f_process_msg=9
  else
    f_process_msg=$2
  fi
  
  if [ -z "$3" -o "$3" != "quiet" ]; then
    f_msgs=1
  else
    f_msgs=0
  fi
  
  # ps -aeo pid,user,args --no-headers
  f_pid="`ps -aeo pid,args --no-headers | grep ${f_process_name} | grep -v 'grep' | gawk '{printf $1 " "}'`"

  if [ -z "${f_pid}" ]; then
    lognl "kill_process_name: process id not found. No such a process (${f_pid}) running. Stop."
    return 1
  fi
  
  #kill the process 
  if [ "${f_msgs}" == "0" ]; then
    kill -${f_process_msg} ${f_pid} >/dev/null 2>&1
  else
    kill -${f_process_msg} ${f_pid}
  fi

  # check that process no longer exists
  if [ "$?" != "0" ]; then
    lognl "kill_process_name: process (${f_process_name}-${f_pid}) not killed"
    return 1
  else
    lognl "kill_process_name: process (${f_process_name}-${f_pid}) killed"
  fi
  return 0
}


# ###########################################################################
# kill_process_id ( process_id [, kill signal] [, quiet] )
#   performs process kill
#   ex. kill_process_id 2323 8 -> kill -8 2323
# ###########################################################################
function kill_process_id ()
{
  # assign kill_process_id arguments
  f_quiet="$(echo "$*" | awk '{printf(tolower($(NF))=="quiet" ? "1" : "0")}')"
  f_process_id="$(echo "$*" | awk '{s="";p="";for(i=NF;i>0;i--){if($i ~ /^[0-9]+$/){if((i>=(NF-1))&&(($i+0)<35)&&(s=="")){s=($i+0)}else{p= $i " " p}}else{continue}} printf(p)}')"
  
  f_process_msg="$(echo "$*" | awk '{s="";p="";for(i=NF;i>0;i--){if($i ~ /^[0-9]+$/){if((i>=(NF-1))&&(($i+0)<35)&&(s=="")){s=($i+0)}else{p= $i " " p}}else{continue}} printf(s==""?"9":s)}')"
  
  
#  echo "${f_quiet}|${f_process_id}|${f_process_msg}|"
  
  # test pid
  if [ -z "${f_process_id}" ]; then
    lognl "kill_process_id: process id not specified. Stop."
    return 1
  fi
  
  # test signal
  if [ -z "${f_process_msg}" ]; then
    f_process_msg=9
  fi
  
  # kill the process 
  kill -${f_process_msg} ${f_process_id} >/dev/null 2>&1
  ecode="$?"
  
  # check that process no longer exists
  if [ "${ecode}" != "0" ]; then
    if [ "${f_quiet}" == "0" ]; then
      lognl "kill_process_id: process ${f_process_id} not killed!"
    fi
  else
    if [ "${f_quiet}" == "0" ]; then
      lognl "kill_process_id: process ${f_process_id} killed"
    fi
  fi
  
  return ${ecode}
}

# ###########################################################################
# ###########################################################################
#                       Randomization functions
# ###########################################################################
# ###########################################################################


# ###########################################################################
# gen_my_rand ()
#   generate custom random number in <0 ... 16) range
#   output random number is stored in MY_RAND
# ###########################################################################
function gen_my_rand ()
{
#  let "MY_RAND=${RANDOM} / 1024"
  let "MY_RAND=${RANDOM} / 2048"
}

# ###########################################################################
# gen_my_rand_in_range ( min, max )
#   generate custom random number ranged to <0...M) or <m...M) interval
#   output random number is stored in MY_RAND
# ###########################################################################
function gen_my_rand_in_range ()
{
  bash_rand_max=32767
  if [ -z "$1" ]; then
    # no range specified, return the raw rand as provided by bash
    MY_RAND=${RANDOM}
  else
    if [ -z "$2" ]; then
      # <0...M) range specified
      let "MY_RAND=($1*${RANDOM})/${bash_rand_max}"
    else
      # <m...M) range specified
      let "MY_RAND=((($2-$1)*${RANDOM})/${bash_rand_max})+$1"
    fi
  
  fi
}

# ###########################################################################
# gen_my_rand_str ( [length] )
#   generate custom random string
#   output random number is stored in MY_RAND_STR
# ###########################################################################
function gen_my_rand_str ()
{
  
  if [[ -z "$1" || "$1" < "1" ]]; then
    # no range specified, return simple sha1sum / md5sum
    if [ "${RANDOM}" -gt "16386" ]; then
      MY_RAND_STR="$(ps auxw | sha1sum | head -1 | awk '{printf $1}')"
    else
      MY_RAND_STR="$(ps auxw | md5sum | head -1 | awk '{printf $1}')"
    fi
  else
    max_str_len=1024
    tmp_str=""
    
    # determine maximal string length
    if [ "$1" -gt "${max_str_len}" ]; then
      req_str_len="${max_str_len}"
    else
      req_str_len="$1"
    fi
    
    # main loop
    while [ 1 ]; do
      if [ "${RANDOM}" -gt "16386" ]; then
        tmp_str="${tmp_str}$(ps auxw | sha1sum | head -1 | awk '{printf $1}')"
      else
        tmp_str="$(ps auxw | sha1sum | head -1 | awk '{printf $1}')${tmp_str}"
      fi
      # get string length
      str_len="$(echo ${tmp_str} | awk '{printf(length($1))}')"
      
      if [ "${str_len}" -ge "${req_str_len}" ]; then
        MY_RAND_STR="$(echo $tmp_str | gawk -v l=${req_str_len} '{printf(substr($1,1,l))}')"
        break
      fi
    done
    
  fi
}


# ###########################################################################
# ###########################################################################
#                    Status and/or information functions
# ###########################################################################
# ###########################################################################


# ###########################################################################
# get information about memory utilization
# ###########################################################################
function get_memory_status ()
{
  lognl "get_memory_status:Current memory status information:"
  free -t
  return 0
}

# ###########################################################################
# get_cpu_info ()
#   get information about cpu from /proc/cpuinfo
# ###########################################################################
function get_cpu_info ()
{
  lognl "Get CPU information:"
  egrep '(processor|vendor_id|model name|cpu MHz|bogomips|cores)' /proc/cpuinfo | \
    sort -ru | gawk 'BEGIN{cnt=0;str="";} \
    {if(($1=="processor")&&($2==":")){cnt++}else{str=sprintf("%s%s\n",str,$0)}} \
    END{printf("processor(s)    : 0...%d {%d in total}\n%s\n",cnt-1,cnt,str)}'
}

# ###########################################################################
# get_essential_info ()
#   get information about cpu os and core tools
# ###########################################################################
function get_essential_info ()
{
  # get basic information
  lognl "Machine and user info:"
  lognl "    os: $(head -1 /etc/issue)"
  lognl " uname: $(uname -a)"
  lognl "  user: ${USER}"
  lognl "   pwd: ${PWD}"
  
  # get cpu information
  get_cpu_info

  # get tools version information
  lognl "General tools version info:"

  # tool versions
  TOOL_LIST=(gawk bash yum rpm curl wget python)
  TLSW_LIST=(-    -    -   -   -    -    -V)
  TDEF_SW="--version"
  
  for ((i=0;i<${#TOOL_LIST[@]};i++)); do
    echo "  ${TOOL_LIST[i]}: $(which ${TOOL_LIST[i]})"
    if [ "${TLSW_LIST[i]}" == "-" ]; then
      echo "    $(${TOOL_LIST[i]} ${TDEF_SW} | head -1)"
    else
      echo "    $(${TOOL_LIST[i]} ${TLSW_LIST[i]} | head -1)"
    fi
  done

  return 0
}


# ###########################################################################
# get_rhlinux_main_release ()
#   gets red hat linux main release information and returns it
# ###########################################################################
function get_rhlinux_main_release ()
{
  rhlv=0
  rhlv=$(gawk 'BEGIN{n=0;l=1}\
               {if(NR==l){for(i=1;i<=length($0);i++)\
                      {if(substr($0,i,1)~[0-9]){n=substr($0,i,1);break}}}}\
               END{printf(n)}' /etc/issue)
  lognl "get_rhlinux_main_release:Get RHEL main release ... ${rhlv}"
  return ${rhlv}
}


# ###########################################################################
# get_os_name_version ( [verbose] )
#   gets UNIX/LINUX os name, main version and minor version if any
#   all stored in OS_NAME, OS_MAIN_VER, OS_MINOR_VER
# ###########################################################################
function get_os_name_version ()
{
  
  for i_file in  /etc/redhat-release /etc/issue ; do
    if [ -r ${i_file} ]; then
     
      # get os name
      export OS_NAME="$(head -1 ${i_file} | gawk '{for(i=1;i<=NF;i++){f=tolower($i);if((f~/[0-9]/)||(f~/release/)){exit(0)}else{if(i>1){printf " " }printf $i}}}')"
      # get versions
      export OS_MAIN_VER="$(head -1 ${i_file} | sed 's/[^0-9]/ /g' | gawk '{print $1}')"
      export OS_MINOR_VER="$(head -1 ${i_file} | sed 's/[^0-9]/ /g' | gawk '{print $2}')"
     
      # stop browsing files
      break
    fi
  done 
  if [ -n "${OS_NAME}" -a -n "${OS_MAIN_VER}" ]; then
    lognl "get_os_name_version: ${OS_NAME}, ${OS_MAIN_VER}, ${OS_MINOR_VER}"
    return 0
  else
    lognl "get_os_name_version:ERROR:Coud not get os type and/or version info"
    return 1
  fi
}


# ###########################################################################
# ps_inactivity_util (PID, files2watch, INACTIVITY_PERIOD) 
#   get processes information once inactivity is detected
# ###########################################################################
function ps_inactivity_util ()
{
  # get PID parameter
  if [ -z "$1" ]; then
    echo "ps_inactivity_util:No PID argument specified!"
    return 1
  else
    ps $1 >/dev/null 2>&1
    if [ "$?" != "0" ]; then
      echo "ps_inactivity_util:PID argument invalid!"
      return 1
    fi
  fi

  # get files to watch parameter
  if [ -z "$2" ]; then
    echo "ps_inactivity_util:No files to watch specified!"
    return 1
  else
    ps_inactivity_util_ff_result=0
    for ifile in "$2"; do
      if [ ! -e ${ifile} ]; then
        ps_inactivity_util_ff_result=1
      fi
    done
    # check result
    if [ "${ps_inactivity_util_ff_result}" != "0" ]; then
      echo "ps_inactivity_util:Some files are not found!"
      return 1
    fi
  fi

  # get inactivity period parameter
  if [ -z "$3" ]; then
    echo "ps_inactivity_util:No inactivity period given!"
    return 1
  else
    if [ "$3" -le "0" ]; then
      echo "ps_inactivity_util:Invalid inactivity period given! ($3)"
      return 1
    fi
  fi
  
  in_pid="$1"
  in_files="$2"
  in_inactivity_period="$3"
  
  L_SIZE_SNAP="$(stat -c%s ${in_files} | column)"
  N_SIZE_SNAP="$(stat -c%s ${in_files} | column)"
  L_TIME_SNAP="${SECONDS}"
  
  while [ 1 ]; do
    # snap file sizes
    N_SIZE_SNAP="$(stat -c%s ${in_files} | column)"
    
    if [ "${N_SIZE_SNAP}" != "${L_SIZE_SNAP}" ]; then
      # watched files are changed
      L_TIME_SNAP="${SECONDS}"
      # update file size stamp
      L_SIZE_SNAP="${N_SIZE_SNAP}"
    else
      # watched files are the same
      
      if [ "$((${SECONDS}-${L_TIME_SNAP}))" -gt "${in_inactivity_period}" ]; then
        # put header
        echo "----Inactivity detected!----$(date +%Y%m%d_%H%M%S)----BEGIN-CUT"
        
        # run inactivity ps tools
        get_system_state_info
        
        # report processes
        pstree -p $1 | gawk '{print "  " $0}'
        get_processes_info_processes="$(pstree -p $1 | tr -cd [0-9\)] | tr ')' ' ')"
    
        # main loop through processes
        for i_ps in ${get_processes_info_processes}; do
          get_process_info ${i_ps}
        done
        
        # put footer
        echo "----Inactivity detected!----$(date +%Y%m%d_%H%M%S)----END-CUT"
        
        # leave the while and the function
        break
        
      fi
    
    fi
    
    
    # flush buffers
    sync
    
    # loop step
    for ((ps_i_u_loop=0; ps_i_u_loop<10; ps_i_u_loop++)); do
      sleep 1
      # was the exit requested by Exit function?
      if [ -e "${EXITING_SIGNAL_FILE}" ]; then
        break
      fi

    done
    
    # was the exit requested by Exit function?
    if [ -e "${EXITING_SIGNAL_FILE}" ]; then
      break
    fi
    
  done

}


# ############################################################################
# get_system_state_info ()
#   get system information from /proc/*
# ############################################################################
function get_system_state_info ()
{

  echo "System state report:"
  # alpha] uptime
  result=$(cat /proc/uptime)
  echo "uptime:${result}"

  # beta] loadavg
  result=$(cat /proc/loadavg)
  echo "loadavg:${result}"

  # gamma] meminfo
  echo "meminfo:"
  cat /proc/meminfo | gawk '{print "  " $0}'

}

# ############################################################################
# get_process_info ( )
#   get system info including detailed process informations
# ############################################################################
function get_process_info ()
{
  # get PID parameter
  if [ -z "$1" ]; then
    echo "get_process_info:No PID argument detected!"
    return 1
  else
    ps $1 >/dev/null 2>&1
    if [ "$?" != "0" ]; then
      echo "get_process_info:PID argument invalid!"
      return 1
    fi
  fi

  echo "process $1 report:"

  # delta-A] process cmdline
  result=$(cat /proc/$1/cmdline)
  echo "cmdline:${result}"


  # delta-B] process status
  echo "process $1 status: (${result})"
  cat /proc/$1/status | gawk '{print "  " $0}'

  # delta-C] pstack process status
  echo "process $1 pstack dump: (${result})"
  pstack $1 | gawk '{print "  " $0}'

  # delta-D] process status - open files/pipes/sockets
  echo "process $1 open files/pipes/sockets: (${result})"
  ls -l /proc/$1/fd | gawk '{print "  " $0}'

#  # delta-D] process status - open files
#  echo "open files:"
#  lsof -i $1
#
#  # delta-E] process status - open sockets
#  echo "open sockets:"
#  lsof -p $1

}


# ###########################################################################
# get_installed_packages ( [pkg_list_to_compare] )
#   list installed packages to screen and to file linked by ${RHTS_TEST_MRG_INSTALLED_PACKAGES}
#   compare current package list to one stored in 
#   example of use:
#     1st run: get_installed_packages
#     2nd run: get_installed_packages ${RHTS_TEST_MRG_INSTALLED_PACKAGES}
# ###########################################################################
function get_installed_packages ()
{
  rpm -qa > /dev/null 2>&1
  if [ "$?" != "0" ]; then
    # rpm doesn't work well
    return 1
  else
    # rpm works well
    if [ -n "$1" -a -r "$1" ]; then
      tmp_list_fn="${HOME}/get_installed_packages.log.$(date +%Y%m%d_%H%M%S)"
      rpm -qa | sort -u > ${tmp_list_fn}
      lognl "get_installed_packages:Following packages are added/removed:"
      diff $1 ${tmp_list_fn}
    else
      lognl "get_installed_packages:Installed packages are:"
      # print to screen using column
      rpm -qa | sort -u | column
      # store current snapshot for later use
      RHTS_TEST_MRG_INSTALLED_PACKAGES="${HOME}/get_installed_packages.log"
      export RHTS_TEST_MRG_INSTALLED_PACKAGES
      rpm -qa | sort -u > ${RHTS_TEST_MRG_INSTALLED_PACKAGES}
    fi
    return 0
  fi
  
}


# ###########################################################################
# get_current_environment ( [verbose] )
#   lists exported and local environment ###########################################################################
function get_current_environment ()
{
  env > /dev/null 2>&1
  ecode="$?"
  set > /dev/null 2>&1
  if [ "$?" != "0" -o "${ecode}" != "0" ]; then
    # env and/or set doesn't work well
    lognl "get_current_environment:ERROR:Cannot display environment!"
    return 1
  else
    lognl "get_current_environment:Environment seen by every program (env)"
    env | sort
    # check whether we pul also detailed environment - verbose flags
    
    if [[ ("$1" == "verbose" ) || \
          ( -n "${TEST_PARAM_RHTS_TEST_VERBOSE_LEVEL}" && \
            "${TEST_PARAM_RHTS_TEST_VERBOSE_LEVEL}" > "5" ) || \
          ( -n "${TEST_PARAM_RHTS_TEST_VERBOSE_GET_ENV}" && \
            "${TEST_PARAM_RHTS_TEST_VERBOSE_GET_ENV}" != "0" ) ]]; then
      lognl "get_current_environment:Environment seen by current shell (set)"
      set
    fi
    return 0
  fi
  
}


# ###########################################################################
# eof
# ###########################################################################
