#!/bin/bash

# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Frantisek Reznicek <freznice@redhat.com>


# source used MRG test common functions
# ###########################################################################
source ./mrg_test_functions.sh


# global test variables 
# ###########################################################################
VERSION=1.0                  # test version
RESULT=FAIL                  # FAIL unless test explicitly passes
ERR_CNT=0                    # error counter init
TST_CNT=0                    # test count
OUTPUTFILE="$PWD/$(basename $PWD).log"
VERBOSE_LEVEL=${TEST_PARAM_RHTS_TEST_VERBOSE_LEVEL:-1}


# other variables
# ###########################################################################


# Source RHTS environment (. /usr/bin/rhts-environment.sh)
# ###########################################################################
rhts_load_environment


# ###########################################################################
# main - MRG installation or compilation & installation
# ###########################################################################

# ${OUTPUTFILE} logging starts here
{

# gather additional info
# ###########################################################################
if [ -n "${VERBOSE_LEVEL}" -a "${VERBOSE_LEVEL}" != "0" ]; then
  # basic info
  get_essential_info
  
  # get info about installed packages
  get_installed_packages

  # get current environment
  get_current_environment

fi

# set the path to common files & export them using MRG test config file
# ###########################################################################
lognl "Export common MRG test files... access via \${RHTS_TEST_MRG_COMMON_DIR}"
export RHTS_TEST_MRG_COMMON_DIR="${PWD}"
export RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS="${PWD}/mrg_test_functions.sh"
export RHTS_TEST_MRG_COMMON_CONFIG="${HOME}/.mrg_test_config"

# installation / compilation stuff
# ###########################################################################


if [ -n "${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}" ]; then
  # install specified
  if [ -n "${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS}" ]; then
    # install & compile&install together specified -> do nothing
    lognl "MRG Install: Both MRG repofile & MRG source install selected! Skipping both! (Install avoided)"
    # do nothing
  else
    # install specified -> do MRG install from specified yum repofile
    lognl "MRG Install: MRG repofile specified -> Installing MRG from ${TEST_PARAM_RHTS_TEST_MRG_REPOFILE}"
    let "TST_CNT++"
    
    # install additional yum repofile if needed
    install_yum_repofiles
    # MRG installation
    install_mrg_packages
    if [ "$?" != "0" ]; then
      lognl ".MRG Install failed"
      let "ERR_CNT++"
    else
      lognl ".MRG Install succeeded"
      # mark that MRG was installed by variable
      export RHTS_TEST_MRG_INSTALLED=1
    fi
  fi
else
  # no install specified
  if [ -n "${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS}" ]; then
    # compile&install specified -> compiling MRG from source
    lognl "MRG Install: MRG source compile&install selected! (revs:${TEST_PARAM_RHTS_TEST_MRG_COMPILE_REVS})"
    let "TST_CNT++"
    
    # install additional yum repofile if needed
    install_yum_repofiles
    # compile & install mrg from sources 
    compile_mrg
    if [ "$?" != "0" ]; then
      lognl ".MRG Compile&Install failed"
      let "ERR_CNT++"
    else
      lognl ".MRG Compile&Install succeeded"
      # mark that MRG was compiled & installed by variable
      export RHTS_TEST_MRG_COMPILED=1

    fi
  else
    # no option specified -> latest MRG packages install
    lognl "MRG Install: No options specified install latest MRG packages"
    let "TST_CNT++"
    
    # install additional yum repofile if needed
    install_yum_repofiles
    # MRG installation
    install_mrg_packages
    if [ "$?" != "0" ]; then
      lognl ".MRG Install failed"
      let "ERR_CNT++"
    else
      lognl ".MRG Install succeeded"
      # mark that MRG was installed by variable
      export RHTS_TEST_MRG_INSTALLED=1
    fi
  fi
fi

# further test configuration
# ###########################################################################

# 1] qpidd logging
# qpidd version query to see which logging option to use
if [ -z "${TEST_PARAM_RHTS_TEST_MRG_QPIDD_LOG2FILE_SWITCH}" ]; then
  # get qpidd version
  get_qpidd_version
  
  case ${QPIDD_VER} in
  0.2)
    export RHTS_TEST_MRG_QPIDD_LOG2FILE_SWITCH='--log-output'
    ;;
  *)
    export RHTS_TEST_MRG_QPIDD_LOG2FILE_SWITCH='--log-to-file'
    ;;
  esac
  
else
  export RHTS_TEST_MRG_QPIDD_LOG2FILE_SWITCH="${TEST_PARAM_RHTS_TEST_MRG_QPIDD_LOG2FILE_SWITCH}"
fi


# export test variables to mrg test common config file
# ###########################################################################
#env | grep ^RHTS_TEST_MRG_ | awk '{print "export " $0}' > ${HOME}/.mrg_test_config
env | grep ^RHTS_TEST_MRG_ | \
  awk '{print "export " $0}' > ${RHTS_TEST_MRG_COMMON_CONFIG}
# append translated environments
env | grep ^RHTS_TEST_MRG_TRANSLATE_ | \
  sed 's/RHTS_TEST_MRG_TRANSLATE_//g' | \
  awk '{print "export " $0}' >> ${RHTS_TEST_MRG_COMMON_CONFIG}
env | grep ^TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_ | \
  sed 's/TEST_PARAM_RHTS_TEST_MRG_TRANSLATE_//g' | \
  awk '{print "export " $0}' >> ${RHTS_TEST_MRG_COMMON_CONFIG}


# ###########################################################################
# report results
# ###########################################################################

if [ "${ERR_CNT}" -eq "0" ]; then
  RESULT=PASS
fi

# report results
rhts_report_result ${TEST} ${RESULT} ${ERR_CNT}

} 2>&1 | tee ${OUTPUTFILE}


# ###########################################################################
# end - testing sequence stops here
# ###########################################################################

# ###########################################################################
# eof
# ###########################################################################
